@@ -3,14 +3,11 @@ package SVG::TT::Graph::Bar;
 use strict;
 use Carp;
 use vars qw($VERSION);
-$VERSION = '0.04';
+$VERSION = '0.06';
 
 use SVG::TT::Graph;
 use base qw(SVG::TT::Graph);
 
-# Nasty global! to store the template in, someone patch
-# please, so that reading from __DATA__ works even if you
-# do it twice from the same script!
 my $template;
 
 =head1 NAME
@@ -35,7 +32,7 @@ SVG::TT::Graph::Bar - Create presentation quality SVG bar graphs easily
 	'title' => 'Sales 2002',
   });
   
-  print "Content-type: image/svg+xml\r\n";
+  print "Content-type: image/svg+xml\r\n\r\n";
   print $graph->burn();
 
 =head1 DESCRIPTION
@@ -65,18 +62,21 @@ title, subtitle etc.
     'width'             => '300',
     'show_data_values'  => 1,
 
-    'scale_divisions'   => '',
     'min_scale_value'           => '0',
 	'stagger_x_labels'  => 0,
+    'rotate_x_labels'   => 0,
 	'bar_gap'           => 1,
 
     'show_x_labels'     => 1,
     'show_y_labels'     => 1,
-
+    'scale_integers'    => 0,
+	'scale_divisions'   => '',
+	
     'show_x_title'      => 0,
     'x_title'           => 'X Field names',
 
     'show_y_title'      => 0,
+    'y_title_text_direction' => 'bt',
     'y_title'           => 'Y Scale',
 
     'show_graph_title'		=> 0,
@@ -84,7 +84,10 @@ title, subtitle etc.
     'show_graph_subtitle'   => 0,
     'graph_subtitle'        => 'Graph Sub Title',
 
-    # Optional - defaults to using embeded stylesheet
+    'key'                   => 0,
+    'key_position'          => 'right',
+
+    # Optional - defaults to using internal stylesheet
     'style_sheet'       => '/includes/graph.css',
   });
 
@@ -143,17 +146,17 @@ scales to fix the space.
 
 =item width()
 
-Set the width of the graph box, this is the total height
+Set the width of the graph box, this is the total width
 of the SVG box created - not the graph it self which auto
 scales to fix the space.
 
 =item style_sheet()
 
 Set the path to an external stylesheet, set to '' if
-you want to revert back to using the defaut embeded version.
+you want to revert back to using the defaut internal version.
 
 To create an external stylesheet create a graph using the
-default embeded version and copy the stylesheet section to
+default internal version and copy the stylesheet section to
 an external file and edit from there.
 
 =item show_data_values()
@@ -170,14 +173,6 @@ is '1', set to '0' if you don't want gaps.
 The point at which the Y axis starts, defaults to '0',
 if set to '' it will default to the minimum data value.
 
-=item scale_divisions()
-
-This defines the gap between markers on the Y axis,
-default is a 10th of the max_value, e.g. you will have
-10 markers on the Y axis. NOTE: do not set this too
-low - you are limited to 999 markers, after that the
-graph won't generate.
-
 =item show_x_labels()
 
 Whether to show labels on the X axis or not, defaults
@@ -189,11 +184,30 @@ This puts the labels at alternative levels so if they
 are long field names they will not overlap so easily.
 Default it '0', to turn on set to '1'.
 
+=item rotate_x_labels()
+
+This turns the X axis labels by 90 degrees.
+Default it '0', to turn on set to '1'.
+
 =item show_y_labels()
 
 Whether to show labels on the Y axis or not, defaults
 to 1, set to '0' if you want to turn them off.
 
+=item scale_integers()
+
+Ensures only whole numbers are used as the scale divisions.
+Default it '0', to turn on set to '1'. This has no effect if 
+scale divisions are less than 1.
+
+=item scale_divisions()
+
+This defines the gap between markers on the Y axis,
+default is a 10th of the max_value, e.g. you will have
+10 markers on the Y axis. NOTE: do not set this too
+low - you are limited to 999 markers, after that the
+graph won't generate.
+
 =item show_x_title()
 
 Whether to show the title under the X axis labels,
@@ -208,6 +222,11 @@ What the title under X axis should be, e.g. 'Months'.
 Whether to show the title under the Y axis labels,
 default is 0, set to '1' to show.
 
+=item y_title_text_direction()
+
+Aligns writing mode for Y axis label. Defaults to 'bt' (Bottom to Top).
+Change to 'tb' (Top to Bottom) to reverse.
+
 =item y_title()
 
 What the title under Y axis should be, e.g. 'Sales in thousands'.
@@ -244,10 +263,10 @@ Where the key should be positioned, defaults to
 
 =head1 NOTES
 
-The default stylesheet handles upto 10 data sets, if you
+The default stylesheet handles upto 12 data sets, if you
 use more you must create your own stylesheet and add the
 additional settings for the extra data sets. You will know
-if you go over 10 data sets as they will have no style and
+if you go over 12 data sets as they will have no style and
 be in black.
 
 =head1 EXAMPLES
@@ -272,7 +291,9 @@ Leo Lapworth (LLAP@cuckoo.org)
 L<SVG::TT::Graph>,
 L<SVG::TT::Graph::Line>,
 L<SVG::TT::Graph::BarHorizontal>,
+L<SVG::TT::Graph::BarLine>,
 L<SVG::TT::Graph::Pie>,
+L<SVG::TT::Graph::TimeSeries>
 
 =cut
 
@@ -309,12 +330,16 @@ sub _set_defaults {
 
 	    'show_x_labels'     => 1,
 		'stagger_x_labels'	=> 0,
+        'rotate_x_labels'   => 0,
+		
 	    'show_y_labels'     => 1,
+        'scale_integers'    => 0,
 
 	    'show_x_title'      => 0,
 	    'x_title'           => 'X Field names',
 	
 	    'show_y_title'      => 0,
+        'y_title_text_direction' => 'bt',
 	    'y_title'           => 'Y Scale',
 	
 	    'show_graph_title'		=> 0,
@@ -354,6 +379,7 @@ __DATA__
 <defs>
 <style type="text/css">
 <![CDATA[
+/* Copy from here for external style sheet */
 .svgBackground{
 	fill:#ffffff;
 }
@@ -414,8 +440,6 @@ __DATA__
 
 .yAxisTitle{
 	fill: #ff0000;
-	writing-mode: tb; 
-	glyph-orientation-vertical: 0;
 	text-anchor: middle;
 	font-size: 14px;
 	font-family: "Arial", sans-serif;
@@ -438,61 +462,73 @@ __DATA__
 /* default fill styles for multiple datasets (probably only use a single dataset on this graph though) */
 .key1,.fill1{
 	fill: #ff0000;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 0.5px;	
 }
 .key2,.fill2{
 	fill: #0000ff;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
 .key3,.fill3{
 	fill: #00ff00;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
 .key4,.fill4{
 	fill: #ffcc00;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
 .key5,.fill5{
 	fill: #00ccff;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
 .key6,.fill6{
 	fill: #ff00ff;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
 .key7,.fill7{
 	fill: #00ffff;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
 .key8,.fill8{
 	fill: #ffff00;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
 .key9,.fill9{
 	fill: #cc6666;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
 .key10,.fill10{
 	fill: #663399;
-	fill-opacity: 0.2;
+	fill-opacity: 0.5;
+	stroke: none;
+	stroke-width: 1px;	
+}
+.key11,.fill11{
+	fill: #339900;
+	fill-opacity: 0.5;
+	stroke: none;
+	stroke-width: 1px;	
+}
+.key12,.fill12{
+	fill: #9966FF;
+	fill-opacity: 0.5;
 	stroke: none;
 	stroke-width: 1px;	
 }
@@ -503,6 +539,7 @@ __DATA__
 	font-family: "Arial", sans-serif;
 	font-weight: normal;
 }
+/* End copy for external style sheet */
 ]]>
 </style>
 </defs>
@@ -520,12 +557,18 @@ __DATA__
 	[% y = 0 %]
 	
 	[% char_width = 9 %]
+	[% half_char_height = 2.5 %]
 
 <!-- calc min and max values -->
 	[% min_value = 99999999999 %]
 	[% max_value = 0 %]
 	[% max_key_size = 0 %]
+	[% max_x_label_size = 0 %]
 	[% FOREACH field = config.fields %]
+		[% IF max_x_label_size < field.length %]
+			[% max_x_label_size = field.length %]
+		[% END %]
+		
 		[% FOREACH dataset = data %]
 			[% IF min_value > dataset.data.$field && dataset.data.$field != '' %]
 				[% min_value = dataset.data.$field %]
@@ -533,7 +576,6 @@ __DATA__
 			[% IF max_value < dataset.data.$field && dataset.data.$field != '' %]
 				[% max_value = dataset.data.$field %]
 			[% END %]
-		<!-- find largest dataset title for Key size -->
 			[% IF max_key_size < dataset.title.length %]
 				[% max_key_size = dataset.title.length %]
 			[% END %]
@@ -541,13 +583,17 @@ __DATA__
 	[% END %]
 
 
-	
-	
-
 <!-- CALC HEIGHT AND Y COORD DIMENSIONS -->
 	<!-- reduce height of graph area if there is labelling on x axis -->
 	[% IF config.show_x_labels %][% h = h - 20 %][% END %]
 	
+	<!-- reduce height if x labels are rotated -->
+	[% max_x_label_length = 0 %]
+	[% IF config.rotate_x_labels %]
+		[% max_x_label_length = max_x_label_size * char_width %]
+		[% h = h - max_x_label_length %]
+	[% END %]
+	
 	<!-- stagger x labels if overlapping occurs -->
 	[% stagger = 0 %]
 	[% IF config.stagger_x_labels %]
@@ -568,9 +614,15 @@ __DATA__
 	
 <!-- reduce graph dimensions if there is a KEY -->
 	[% key_box_size = 12 %]
+	[% key_padding = 5 %]
 	
 	[% IF config.key && config.key_position == 'right' %][% w = w - (max_key_size * (char_width - 1)) - (key_box_size * 3 ) %]
-	[% ELSIF config.key && config.key_position == 'bottom' %][% h = h - 50 %]
+	[% ELSIF config.key && config.key_position == 'bottom' %]
+        [% IF data.size < 4 %]
+            [% h = h - ((data.size + 1) * (key_box_size + key_padding))%]
+        [% ELSE %]
+            [% h = h - (4 * (key_box_size + key_padding))%]
+        [% END %]        
 	[% END %]
 
 	<!-- find start value for scale on y axis -->
@@ -597,9 +649,17 @@ __DATA__
 	[% IF config.scale_divisions %]
 		[% scale_division = config.scale_divisions %]
 	[% ELSE %]
-		[% scale_division = scale_range / 10 FILTER format('%2.0f') %]
+		[% scale_division = scale_range / 10 FILTER format('%2.01f') %]
 	[% END %]
-
+	
+	[% IF config.scale_integers %]
+		[% IF scale_division < 1 %]
+			[% scale_division = 1 %]
+		[% ELSIF scale_division.match('.') %]
+			[% scale_division = scale_division FILTER format('%2.0f') %]
+		[% END %]
+	[% END %]
+	
 <!-- JUMP THE GUN AND CALC BAR WIDTHS AS THESE ARE USED FOR PADDING IF LARGE X LABELS ARE USED -->
 <!-- get number of data points on x scale -->
 [% dx = config.fields.size %]
@@ -637,11 +697,11 @@ __DATA__
 	[% IF config.show_y_labels && space_b4_y_axis < max_value_length_px %]
 		<!-- allow slightly more padding if small labels -->
 		[% IF max_value_length < 2 %]
-			[% w = w - (max_value_length * (char_width * 2)) %]
-			[% x = x + (max_value_length * (char_width * 2)) %]
+			[% w = w - (max_value_length * (char_width * 2)) - char_width %]
+			[% x = x + (max_value_length * (char_width * 2)) + char_width %]
 		[% ELSE %]
-			[% w = w - max_value_length_px %]
-			[% x = x + max_value_length_px %]
+			[% w = w - max_value_length_px + char_width %]
+			[% x = x + max_value_length_px + char_width %]
 		[% END %]
 	[% END %]
 	
@@ -663,11 +723,9 @@ __DATA__
 	<path d="M[% x %] [% base_line %] h[% w %]" class="axis" id="yAxis"/>
 
 <!-- //////////////////////////////  AXIS DISTRIBUTIONS //////////////////////////// -->
-	<!-- get number of data points on x scale -->
-	[% dx = config.fields.size %]
-		<!-- ensure x_data_points butt up to edge of graph -->
-		[%# dx = dx - 1 %]
-		[% dx = dx %]
+<!-- get number of data points on x scale -->
+[% dx = config.fields.size %]
+
 <!-- get distribution width on x axis -->
 [% data_widths_x = w / dx %]
 [% dw = data_widths_x.match('(\d+[\.\d\d])').0 %]
@@ -677,6 +735,9 @@ __DATA__
 
 [% IF config.bar_gap %]
 	[% bar_gap = 10 %]
+	[% IF dw < bar_gap %]
+		[% bar_gap = dw / 2 %]
+	[% END %]
 [% ELSE %]
 	[% bar_gap = 0 %]
 [% END %]
@@ -686,14 +747,14 @@ __DATA__
 [% IF config.show_x_labels %]
 	[% FOREACH field = config.fields %]
 		[% IF count == 0 %]
-<text x="[% x + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 %]" class="xAxisLabels">[% field %]</text>
+<text x="[% x + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 %]" [% IF config.rotate_x_labels %]transform="rotate(90 [% x + (dw / 2) - (bar_gap / 2) - half_char_height %] [% base_line + 15 %])" style="text-anchor: start" [% END %]class="xAxisLabels">[% field %]</text>
 		[% i = i - dw %]
 		[% ELSE %]
 			[% IF stagger_count == 2 %]
-				<text x="[% x + i + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 %]" class="xAxisLabels">[% field %]</text>
+				<text x="[% x + i + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 %]" [% IF config.rotate_x_labels %]transform="rotate(90 [% x + i + (dw / 2) - (bar_gap / 2) - half_char_height %] [% base_line + 15 %])" style="text-anchor: start" [% END %]class="xAxisLabels">[% field %]</text>
 				[% stagger_count = 0 %]
 			[% ELSE %]
-				<text x="[% x + i + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 + stagger %]" class="xAxisLabels">[% field %]</text>
+				<text x="[% x + i + (dw / 2) - (bar_gap / 2) %]" y="[% base_line + 15 + stagger %]" [% IF config.rotate_x_labels %]transform="rotate(90 [% x + i + (dw / 2) - (bar_gap / 2) - half_char_height %] [% base_line + 15 + stagger %])" style="text-anchor: start" [% END %]class="xAxisLabels">[% field %]</text>
 				<path d="M[% x + i + (dw / 2) - (bar_gap / 2) %] [% base_line %], v[% stagger %]" class="staggerGuideLine" />
 			[% END %]
 		[% END %]
@@ -727,7 +788,6 @@ __DATA__
 
 
 
-
 <!-- //////////////////////////////  AXIS TITLES ////////////////////////////// -->
 
 <!-- x axis title -->
@@ -737,12 +797,16 @@ __DATA__
 		[% ELSE %]
 			[% y_xtitle = 35 %]
 		[% END %]
-		<text x="[% (w / 2) + x %]" y="[% h + y + y_xtitle + stagger %]" class="xAxisTitle">[% config.x_title %]</text>
+		<text x="[% (w / 2) + x %]" y="[% h + y + y_xtitle + stagger + max_x_label_length %]" class="xAxisTitle">[% config.x_title %]</text>
 	[% END %]	
 
 <!-- y axis title -->
 	[% IF config.show_y_title %]
-			<text x="10" y="[% (h / 2) + y %]" class="yAxisTitle">[% config.y_title %]</text>
+		[% IF config.y_title_text_direction == 'tb' %]
+			<text x="11" y="[% (h / 2) + y %]" class="yAxisTitle" style="writing-mode:tb;">[% config.y_title %]</text>
+		[% ELSE %]
+			<text class="yAxisTitle" transform="translate(15,[% (h / 2) + y %]) rotate(270)">[% config.y_title %]</text>
+		[% END %]
 	[% END %]
 
 
@@ -753,7 +817,7 @@ __DATA__
 
 [% divider = dy / scale_division %]
 <!-- data points on graph -->
-[% IF config.show_data_points || config.show_data_values %]
+
 	[% xcount = 0 %]
 	
 	[% FOREACH field = config.fields %]
@@ -769,7 +833,7 @@ __DATA__
 		[% END %]
 		[% xcount = xcount + 1 %]		
 	[% END %]
-[% END %]
+
 
 <!-- //////////////////////////////// KEY /////// ////////////////////////// -->
 [% key_count = 1 %]
@@ -783,8 +847,13 @@ __DATA__
 [% ELSIF config.key && config.key_position == 'bottom' %]
 	<!-- calc y position of start of key -->
 	[% y_key = base_line %]
-	[% IF config.show_x_labels %][% y_key = base_line + 20 %][% END %]
-	[% IF config.show_x_title %][% y_key = base_line + 25 %][% END %]
+	[% IF config.show_x_title %][% y_key = y_key + 25 %][% END %]
+	[% IF config.rotate_x_labels && config.show_x_labels %]
+		[% y_key = y_key + max_x_label_length %]
+	[% ELSIF config.show_x_labels && stagger < 1 %]
+		[% y_key = y_key + 20 %]
+	[% END %]
+	
 	[% y_key_start = y_key %]
 	[% x_key = x %]
 	[% FOREACH dataset = data %]
@@ -793,9 +862,9 @@ __DATA__
 			[% x_key = x_key + 200 %]
 			[% y_key = y_key - (key_box_size * 4) - 2 %]
 		[% END %]
-		<rect x="[% x_key %]" y="[% y_key + (key_box_size * key_count) + (key_count * key_padding) %]" width="[% key_box_size %]" height="[% key_box_size %]" class="key[% key_count %]"/>
+			<rect x="[% x_key %]" y="[% y_key + (key_box_size * key_count) + (key_count * key_padding) + stagger %]" width="[% key_box_size %]" height="[% key_box_size %]" class="key[% key_count %]"/>
 
-		<text x="[% x_key + key_box_size + key_padding %]" y="[% y_key + (key_box_size * key_count) + (key_count * key_padding) + key_box_size %]" class="keyText">[% dataset.title %]</text>
+			<text x="[% x_key + key_box_size + key_padding %]" y="[% y_key + (key_box_size * key_count) + (key_count * key_padding) + key_box_size + stagger %]" class="keyText">[% dataset.title %]</text>
 		[% key_count = key_count + 1 %]
 	[% END %]
 	
